# MCollective Puppet Enterprise
#
# This module manages the initial configuration of MCollective for use with
# Puppet Enterprise.  The primary purpose is to generate RSA key pairs for the
# initial "mco" user account on the Puppet Master and a shared set of RSA keys
# for all of the MCollective server processes (Puppet Agent roles).
#
# Resources managed on a Puppet Master:
#
#  * mco user account
#  * RSA keys to identify and authorize the mco user.
#  * One set of RSA keys generated for shared use among all MCollective agents.
#
# Resources managed on a Puppet Agent:
#
#  * RSA keys for MCollective service
#  * mco user account public RSA key to authenticate the mco user
#
# == Parameters
#
# This class expects four facts to be provided:
#
#  * fact_is_puppetmaster (true/false)
#  * fact_is_puppetagent  (true/false)
#  * fact_stomp_server    (hostname)
#  * fact_stomp_port      (TCP port number)
#
# The facts are automatically set by the Puppet Enterprise installer for each
# system.
class mcollectivepe {

  validate_re($fact_is_puppetmaster, '^true$|^false$')
  validate_re($fact_is_puppetagent, '^true$|^false$')
  validate_re($fact_stomp_server, '^[a-zA-Z]\w*')
  validate_re($fact_stomp_port, '^[0-9]+$')

  # Convert facter strings to booleans
  $is_puppetmaster = $fact_is_puppetmaster ? { 'true'  => true, 'false' => false }
  $is_puppetagent = $fact_is_puppetagent ? { 'true'  => true, 'false' => false }

  $stomp_server = $fact_stomp_server ? {
    undef   => 'puppetmaster',
    default => $fact_stomp_server,
  }
  $stomp_port = $fact_stomp_port ? {
    undef   => '61613',
    default => $fact_stomp_port,
  }

  if $is_puppetmaster {
    # Make sure ActiveMQ is running on the master.
    service { 'pe-activemq':
      ensure     => running,
      hasstatus  => true,
      hasrestart => true,
    }
    # Make sure we have a shared certificate for all of the MC servers
    exec { 'mcollective-server-cert':
      command => 'puppet cert --generate mcollective',
      path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
      creates => '/etc/puppetlabs/puppet/ssl/certs/mcollective.pem',
      before  => [ File['mcollective-public.pem'], File['mcollective-private.pem'], File['mcollective-cert.pem'] ],
    }
    exec { 'mcollective-client-cert':
      command => 'puppet cert --generate mco',
      path    => '/opt/puppet/bin:/usr/kerberos/sbin:/usr/kerberos/bin:/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin',
      creates => '/etc/puppetlabs/puppet/ssl/certs/mco.pem',
      before  => [ File['mco-public.pem'] ]
    }
  }

  # Configure the SSL keys on each mcollective server
  file { '/etc/puppetlabs/mcollective/ssl':
    ensure => directory,
    owner  => '0',
    group  => '0',
    mode   => '0755',
  }
  # The use of the selector is because we can't use the file() function in a single
  # run on the puppet master.  The file won't exist until the catalog is applied.
  file { 'mcollective-public.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/mcollective-public.pem',
    ensure  => file,
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/public_keys/mcollective.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/public_keys/mcollective.pem'),
    },
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
  }
  # The use of the selector is because we can't use the file() function in a single
  # run on the puppet master.  The file won't exist until the catalog is applied.
  file { 'mcollective-private.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/mcollective-private.pem',
    ensure  => file,
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/private_keys/mcollective.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/private_keys/mcollective.pem'),
    },
    owner   => '0',
    group   => '0',
    mode    => '0600',
    notify  => Service['mcollective'],
  }
  # The use of the selector is because we can't use the file() function in a single
  # run on the puppet master.  The file won't exist until the catalog is applied.
  file { 'mcollective-cert.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/mcollective-cert.pem',
    ensure  => file,
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/certs/mcollective.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/certs/mcollective.pem'),
    },
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
  }
  file { '/etc/puppetlabs/mcollective/ssl/clients':
    ensure => directory,
    owner  => '0',
    group  => '0',
    mode   => '0755',
  }
  file { 'mco-public.pem':
    path    => '/etc/puppetlabs/mcollective/ssl/clients/mco-public.pem',
    source  => $is_puppetmaster ? {
      true  => '/etc/puppetlabs/puppet/ssl/public_keys/mco.pem',
      false => undef,
    },
    content => $is_puppetmaster ? {
      true  => undef,
      false => file('/etc/puppetlabs/puppet/ssl/public_keys/mco.pem'),
    },
    ensure  => 'file',
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
  }
  # All servers need the shared public key to decrypt and encrypt
  # registration messages.
  file { '/etc/puppetlabs/mcollective/ssl/clients/mcollective-public.pem':
    ensure  => file,
    source  => '/etc/puppetlabs/mcollective/ssl/mcollective-public.pem',
    owner   => '0',
    group   => '0',
    mode    => '0644',
    notify  => Service['mcollective'],
    require => File['mcollective-public.pem'],
  }

  # Manage the MCollective server configuration
  file { '/etc/puppetlabs/mcollective/server.cfg':
    ensure  => file,
    content => template("${module_name}/server.cfg.erb"),
    owner   => 0,
    group   => 0,
    mode    => 0600,
    notify  => Service['mcollective'],
  }
  service { 'mcollective':
    ensure     => running,
    name       => 'pe-mcollective',
    enable     => true,
    hasstatus  => true,
    hasrestart => true,
  }

  # The Puppet Master is the only system to get the mco user account
  # and user-specific encryption keys.
  if $is_puppetmaster {
    accounts::user { 'mco':
      ensure   => present,
      password => '!!',
      home     => '/var/lib/mco',
    }
    file { '/var/lib/mco/.mcollective':
      ensure  => file,
      content => template('mcollectivepe/client_mco.cfg.erb'),
      owner   => 'mco',
      group   => 'mco',
      mode    => '0600',
      require => Accounts::User['mco'],
    }
    file { '/var/lib/mco/.mcollective.d':
      ensure  => directory,
      owner   => 'mco',
      group   => 'mco',
      mode    => '0700',
      require => Accounts::User['mco'],
    }
    file { '/var/lib/mco/.mcollective.d/mco-private.pem':
      ensure  => file,
      source  => '/etc/puppetlabs/puppet/ssl/private_keys/mco.pem',
      owner   => 'mco',
      group   => 'mco',
      mode    => '0600',
      require => [ Accounts::User['mco'], Exec['mcollective-client-cert'] ],
    }
    file { '/var/lib/mco/.mcollective.d/mco-public.pem':
      ensure  => file,
      source  => '/etc/puppetlabs/puppet/ssl/public_keys/mco.pem',
      owner   => 'mco',
      group   => 'mco',
      mode    => '0644',
      require => [ Accounts::User['mco'], Exec['mcollective-client-cert'] ],
    }
    file { '/var/lib/mco/.mcollective.d/mco-cert.pem':
      ensure  => file,
      source  => '/etc/puppetlabs/puppet/ssl/certs/mco.pem',
      owner   => 'mco',
      group   => 'mco',
      mode    => '0644',
      require => [ Accounts::User['mco'], Exec['mcollective-client-cert'] ],
    }
  }

  # (#8826) Manage the plugins for MCollective in Puppet Enterprise
  # The anchors are to gurantee resource containment for the module
  anchor { 'mcollectivepe::begin': }
  class { 'mcollectivepe::plugins':
    require => Anchor['mcollectivepe::begin'],
    before  => Anchor['mcollectivepe::end'],
  }
  # (#9045) Update facter facts on disk periodically using a cron job.
  # This is a separate class because I'm anticipating platform oddities
  # with cron on different platforms.
  class { 'mcollectivepe::metadata':
    require => Anchor['mcollectivepe::begin'],
    before  => Anchor['mcollectivepe::end'],
  }
  anchor { 'mcollectivepe::end': }

}
